﻿using System;
using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.Server;
using Vintagestory.API.Common.Entities;
using Vintagestory.GameContent;
using Vintagestory.API.MathTools;
using Vintagestory.API.Util;

[assembly: ModInfo( "FlintAndSteel",
	Description = "Class file which changes the way the Flint and Steel functions.",
	Website     = "",
	Authors     = new []{ "OmegaHaxors" } )]

namespace FlintAndSteel
{
	public class FlintAndSteel: ModSystem
	{
		ICoreAPI api = null;
		ICoreServerAPI sapi = null;
		ICoreClientAPI capi = null;

		public override void Start(ICoreAPI api)
		{
			this.api = api;
			api.RegisterItemClass("ItemFlintAndSteel",typeof(ItemFlintAndSteel));
			api.RegisterEntity("EntitySpark",typeof(EntitySpark));
		}
		public override void StartClientSide(ICoreClientAPI capi){this.capi = capi;}
		public override void StartServerSide(ICoreServerAPI sapi)
		{
			this.sapi = sapi;
		}
	}
	public class EntitySpark : EntityAgent
	{	
		public override void Initialize(EntityProperties properties, ICoreAPI api, long InChunkIndex3d)
		{
			base.LightHsv = new Byte[]{10,7,5};
			base.Initialize(properties, api, InChunkIndex3d);
		}
	}
	public class ItemFlintAndSteel : ItemFirestarter
	{
		public override void OnHeldInteractStart(ItemSlot slot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel, bool firstEvent, ref EnumHandHandling handling)
		{
			base.OnHeldInteractStart(slot,byEntity,blockSel,entitySel,firstEvent,ref handling);
			if (this.api.Side == EnumAppSide.Client){this.api.Event.UnregisterCallback(ObjectCacheUtil.TryGet<long>(this.api, "firestartersound"));}
		}
		public override bool OnHeldInteractStep(float secondsUsed, ItemSlot slot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel)
		{
			secondsUsed = 1000f;
			return base.OnHeldInteractStep(secondsUsed, slot, byEntity, blockSel, entitySel);
		}
		public override void OnHeldInteractStop(float secondsUsed, ItemSlot slot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel)
		{
			secondsUsed = 1000f;
			if(api.Side == EnumAppSide.Server)
			{
			api.World.PlaySoundAt(new AssetLocation("flintandsteelmod:sounds/ignite"),byEntity,null,true,32f,1f);
			}
			EntityProperties type = api.World.GetEntityType(new AssetLocation("flintandsteelmod:spark-flintandsteel"));
			Entity spark = api.ClassRegistry.CreateEntity(type);
			Vec3d spawnloc = new Vec3d(blockSel.Position.X+0.5,blockSel.Position.Y+0.5,blockSel.Position.Z+0.5);
			spark.ServerPos.SetPos(spawnloc);
			spark.ServerPos.SetYaw(0f);
			spark.Pos.SetFrom(byEntity.ServerPos);
			spark.PositionBeforeFalling.Set(spawnloc);
			api.World.SpawnEntity(spark);
			BlockPos pos = blockSel.Position;
			Random rnd = api.World.Rand;
			api.World.SpawnParticles(new SimpleParticleProperties
				{
					MinQuantity = 5f,
					AddQuantity = 10f,
					Color = ColorUtil.ToRgba(127, 255, 255, 0),
					MinPos = new Vec3d(pos.X + 0.5, pos.Y + 0.5625 + 0.09999999776482582, pos.Z + 0.5),
					AddPos = new Vec3d(0.0625, 0.009999999776482582, 0.0625),
					MinVelocity = new Vec3f(0f, 1f, 0f),
					AddVelocity = new Vec3f(4f * ((float)rnd.NextDouble() - 0.5f), 1f * ((float)rnd.NextDouble() - 0.5f), 4f * ((float)rnd.NextDouble() - 0.5f)),
					LifeLength = 0.2f,
					GravityEffect = 0.75f,
					MinSize = 0.1f,
					MaxSize = 0.4f,
					ParticleModel = EnumParticleModel.Cube,
					SizeEvolve = new EvolvingNatFloat(EnumTransformFunction.LINEAR, -0.15f)
				}, null);
			base.OnHeldInteractStop(secondsUsed,slot,byEntity,blockSel,entitySel);
		}

		public override WorldInteraction[] GetHeldInteractionHelp(ItemSlot inSlot)
		{
			return new WorldInteraction[]
			{
				new WorldInteraction
				{
					HotKeyCode = "shift",
					ActionLangCode = "heldhelp-igniteblock",
					MouseButton = EnumMouseButton.Right
				}
			};
		}
	}
}